#![allow(unused_macros, unused_imports, dead_code)]

pub mod report; pub use report::Report;

macro_rules! byte_code {
    ($($id:ident, $value:expr),* $(,)?) => {
        $(pub const $id :u8 = $value;)*
    };
}

macro_rules! hid_report_descriptor {
    ($name:ident, $($val:expr),* $(,)?) => {
        pub const $name: &[u8] = &[$($val),*];
    };
}

// Global Items
byte_code! {
    USAGE_PAGE,     0x05,
    USAGE,          0x09,
    COLLECTION,     0xA1,
    END_COLLECTION, 0xC0,
    
    LOGICAL_MIN,    0x15,
    LOGICAL_MIN2,   0x16,
    LOGICAL_MIN4,   0x17,
    LOGICAL_MAX,    0x25,
    LOGICAL_MAX2,   0x26,
    LOGICAL_MAX4,   0x27,
    PHYSICAL_MIN,   0x35,
    PHYSICAL_MIN2,  0x36,
    PHYSICAL_MIN4,  0x37,
    PHYSICAL_MAX,   0x45,
    PHYSICAL_MAX2,  0x46,
    PHYSICAL_MAX4,  0x47,
    EXPONENT,       0x55,
    EXPONENT2,      0x56,
    EXPONENT4,      0x57,
    UNIT,           0x65,
    UNIT2,          0x66,
    UNIT4,          0x67,
    
    REPORT_ID,      0x85,
    REPORT_SIZE,    0x75,
    REPORT_COUNT,   0x95,
    
    INPUT,          0x81,
    INPUT2,         0x82,
    INPUT4,         0x83,
    OUTPUT,         0x91,
    OUTPUT2,        0x92,
    OUTPUT4,        0x93,
    
    USAGE_MIN,      0x19,
    USAGE_MAX,      0x29,
}

// Collection Types
byte_code! {
    COLLECTION_APPLICATION, 0x01,
    COLLECTION_PHYSICAL,    0x00,
}

// Input/Output Flags
byte_code! {
    DATA,      0x00,
    CONST,     0x01,
    ARRAY,     0x00,
    VAR,       0x02,
    ABS,       0x00,
    REL,       0x04,
    WRAP,      0x08,
    NONLINEAR, 0x10,
    NOPREF,    0x20,
    NULL,      0x40,
    VOLATILE,  0x80,
}

// Usage Pages
byte_code!(
    U_DESKTOP,  0x01,
    U_KEYBOARD, 0x07,
    U_BUTTON,   0x09,
);

// Collection Usages
byte_code!(
    US_POINTER,   0x01,
    US_MOUSE,     0x02,
    US_JOYSTICK,  0x04,
    US_GAMEPAD,   0x05,
    US_KEYBOARD,  0x06,
    US_KEYPAD,    0x07,
    US_MULTIAXIS, 0x08,
);

// Value Usages
byte_code!(
    US_X,      0x30,
    US_Y,      0x31,
    US_Z,      0x32,
    US_RX,     0x33,
    US_RY,     0x34,
    US_RZ,     0x35,
    US_SLIDER, 0x36,
    US_DIAL,   0x37,
    US_WHEEL,  0x38,
    US_HAT,    0x39,
    US_START,  0x3d,
    US_SELECT, 0x3e,
    US_VX,     0x40,
    US_VY,     0x41,
    US_VZ,     0x42,
    UX_VBX,    0x43,
    US_VBY,    0x44,
    US_VBZ,    0x45,
    US_VNO,    0x46,
    US_QX,     0x49,
    US_QY,     0x4a,
    US_QZ,     0x4b,
    US_QW,     0x4c,
    US_DUP,    0x90,
    US_DDOWN,  0x91,
    US_DRIGHT, 0x92,
    US_DLEFT,  0x93,
);

pub(crate) use hid_report_descriptor;
